/*******************************************************************************
*  XIAOrtcset.ino - XIAOのPCF8563(RTC)の時間を設定するスケッチ                 *
*                                                                              *
*  メモ：このスケッチは、Seeeduino XIAO用の機能拡張ボードに搭載しているRTCの   *
*        日付時刻を設定する物です。                                            *
*        スイッチを長く押すと次の時間設定に移動します。                        *
*        短く押すと点滅している数字(0-9)が切り変わります。                     *
*        日付時刻曜日を全部入力すると[OK]{NO]が現れます、決定は短押しで切替えは*
*        長押しです。間違った入力時は[NO]を選び最初から入れ直しです。          *
*        [OK]で時間がRTCに書き込まれ、時間が表示更新します、この間にボタンを   *
*        押すと再度変更できます。                                              *
* ============================================================================ *
*   VERSION  DATE        BY             CHANGE/COMMENT                         *
* ---------------------------------------------------------------------------- *
*   1.00     2021-01-29  きむ茶工房     Create                                 *
* ============================================================================ *
*   Arduino IDE 1.8.12 (Ardino Duemilanove 328/UNO/Seeeduino XIAO)             *
*******************************************************************************/
#include <U8g2lib.h>
#include <PCF8563.h>
#include <skPCF8563.h>

char *WeekDayData[] = {"Sun","Mon","Tue","Wed","Thu","Fri","Sat"} ;

// RTC(PCF8563)を使用する為のコンストラクタ
PCF8563   rtc ;
skPCF8563 skRTC ;
// SSD1306をグラフィックディスプレイとして使用する為のコンストラクタ
// メモリバッファ(RAM)は128X64バイト(画面フルサイズ分)
// ローテーション(画面回転)なし、SCL=5番ピン　SDA=4番ピン
U8G2_SSD1306_128X64_NONAME_F_HW_I2C u8g2(U8G2_R0,5,4) ;

uint8_t ProcFlag   = 0 ;    // 処理の情報
uint8_t BtnState   = 0;     // 押しボタンステータスを読み取る為の変数
uint8_t BtnCount   = 0 ;    // 押している間の時間のカウント変数
uint8_t OkNoFlag   = 0 ;    // [OK][NO]選択情報変数
String  DateBuffer = "2000/01/01" ;
uint8_t DataFlag   = 0 ;    // 日付数字の点滅状態フラグ
uint8_t DatePos    = 2 ;    // DateBufferの文字位置
String  TimeBuffer = "01:00:00" ;
uint8_t TimeFlag   = 0 ;    // 時刻数字の点滅状態フラグ
int8_t  TimePos    = -1 ;   // TimeBufferの文字位置
uint8_t WeekDay ;           // 曜日文字列の位置情報
uint8_t WeekDayFlag = 0 ;   // 曜日文字列の点滅状態フラグ
char    WorkBuffer[12] ;

////////////////////////////////////////////////////////////////////////////////
// 初期化の処理
void setup() {
    // 押しボタンピン(1番)を入力として初期化します：
    pinMode(1,INPUT_PULLUP) ;
    // RTCの初期化
    rtc.init() ;
    // OLEDの初期化
    u8g2.begin() ;
    // 現在の時刻をＲＴＣから得る
    GetTime() ;
}

////////////////////////////////////////////////////////////////////////////////
// メインのループ処理
void loop() {
    String buff ;
    char   c ;

    BtnState = digitalRead(1) ;  // 押されたらLOW
    if (ProcFlag != 9) {
        /********************/
        /* 押しボタンの処理 */
        /********************/
        if (BtnState == LOW) BtnCount++ ; // 押されている間時間をカウント
        if (BtnCount != 0 && BtnState == HIGH) {
            // ボタンが離された時の処理
            if (BtnCount >= 5) {   // 500ms(5x100ms)を変えたい場合はこの数値を変更
                // 500ms程長押しされた場合は次の数値入力に進む
                // 次の日付へ進む
                DatePos++ ;
                if (DatePos==4 || DatePos==7) DatePos++ ;
                if (DatePos>=10) {
                    DatePos = 10 ;
                    // 日付は終了次は時刻へ進む
                    TimePos++ ;
                    if (TimePos==2 || TimePos==5) TimePos++ ;
                    if (TimePos>=8) TimePos = 8 ;
                }
                BtnState = 98 ;    // 長押し
            } else BtnState = 99 ; // 短押し
            BtnCount = 0 ;         // カウントクリア
        }
    } else {
        /************************/
        /* 現在の時刻を表示する */
        /************************/
        // 現在の時刻をＲＴＣから得る
        GetTime() ;
        // ボタンが押されたら再度日付時刻の設定に戻る
        if (BtnState == LOW) {
            OkNoFlag = 0 ;
            TimePos  = -1 ;
            DatePos  = 2 ;
            ProcFlag = 0 ;
            delay(500) ;
            return ;
        }
    }
    // バッファのクリア
    u8g2.clearBuffer();
    // 17Pixel数字文字で等幅フォント
    u8g2.setFont(u8g2_font_t0_22b_mn) ;
    /**************/
    /* 日付の処理 */
    /**************/
    buff = DateBuffer ;
    if (DatePos < 10 && ProcFlag != 9) {
        // 日付の入力中
        if (BtnState == 99) {
            // 短く押された場合は次の選択候補数字(0-9)に切り替える
            c = DateBuffer.charAt(DatePos)+1 ;
            if (c > '9') c = '0' ;
            DateBuffer.setCharAt(DatePos, c) ;
        }
        // 数字を点滅させる処理
        if (DataFlag == 0) {
            // 数字を消す
            buff.setCharAt(DatePos, ' ') ;
            DataFlag = 1 ;
        } else {
            // 数字を表示する
            buff.setCharAt(DatePos, DateBuffer.charAt(DatePos)) ;
            DataFlag = 0 ;
        }
    }
    // 日付を表示させる
    u8g2.setCursor(0, 17) ;
    u8g2.print(buff) ;
    /**************/
    /* 時刻の処理 */
    /**************/
    buff = TimeBuffer ;
    if (TimePos < 8 && DatePos == 10 && ProcFlag != 9) {
        // 日付は終わり時刻の入力中
        if (BtnState == 99) {
            // 短く押された場合は次の選択候補数字(0-9)に切り替える
            c = TimeBuffer.charAt(TimePos)+1 ;
            if (c > '9') c = '0' ;
            TimeBuffer.setCharAt(TimePos, c) ;
        }
        // 数字を点滅させる処理
        if (TimeFlag == 0) {
            // 数字を消す
            buff.setCharAt(TimePos, ' ') ;
            TimeFlag = 1 ;
        } else {
            // 数字を表示する
            buff.setCharAt(TimePos, TimeBuffer.charAt(TimePos)) ;
            TimeFlag = 0 ;
        }
    }
    // 時刻を表示させる
    u8g2.setCursor(15, 33) ;
    u8g2.print(buff) ;
    /**************/
    /* 曜日の処理 */
    /**************/
    u8g2.setFont(u8g2_font_ncenB12_tr) ;
    sprintf(WorkBuffer,"(%s)",WeekDayData[WeekDay]) ;
    if (TimePos == 8 && ProcFlag == 1 && ProcFlag != 9) {
        // 時刻の入力は終わった
        if (BtnState == 98) TimePos = 9 ; // 長押し
        if (BtnState == 99) {             // 短押し
            // 次の曜日に進む
            WeekDay++ ;
            if (WeekDay >= 7) WeekDay = 0 ;
            sprintf(WorkBuffer,"(%s)",WeekDayData[WeekDay]) ;
        }
        // 曜日を点滅させる処理
        if (WeekDayFlag == 0) {
            // 曜日を消す
            memset(WorkBuffer,0x20,strlen(WorkBuffer)) ;
            WeekDayFlag = 1 ;
        } else WeekDayFlag = 0 ;
    }
    // 時刻の入力終わった後一度だけ上の処理を省く
    if (TimePos == 8) ProcFlag = 1 ;
    // 曜日を表示させる
    u8g2.setCursor(40, 48) ;
    u8g2.print(WorkBuffer) ;
    /**********************************/
    /* 設定した日付時刻をどうするの？ */
    /**********************************/
    if (TimePos == 9 && ProcFlag != 9) {
        u8g2.setFont(u8g2_font_7x14B_mr) ;
        // [OK]ＲＴＣに書き込む [NO]最初からやり直す
        // [OK][NO]を切り替える処理(ボタンは長押し)
        if (BtnState == 98) OkNoFlag = ~OkNoFlag ;
        if (OkNoFlag == 0) {
            // [OK]を選択
            u8g2.setDrawColor(0);
            u8g2.drawStr(15, 63, "[ OK ]") ;
            u8g2.setDrawColor(1);
            u8g2.drawStr(70, 63, "[ NO ]") ;
        }
        if (OkNoFlag == 0xFF) {
            // [NO]を選択
            u8g2.drawStr(15, 63, "[ OK ]") ;
            u8g2.setDrawColor(0);
            u8g2.drawStr(70, 63, "[ NO ]") ;              
            u8g2.setDrawColor(1);
        }
        if (BtnState == 99) {
            // [OK][NO]を決定した(ボタンは短押し)
            if (OkNoFlag == 0xFF) {
                // [NO]を選択した
                OkNoFlag = 0 ;
                TimePos  = -1 ;
                DatePos  = 2 ;
                ProcFlag = 0 ;
            } else {
                // [OK]を選択した、ＲＴＣに書き込む
                rtc.stopClock() ; // RTCのカウントを止める
                rtc.setYear(DateBuffer.substring(2, 4).toInt()) ;  // 年
                rtc.setMonth(DateBuffer.substring(5, 7).toInt()) ; // 月
                rtc.setDay(DateBuffer.substring(8, 10).toInt()) ;  // 日
                rtc.setHour(TimeBuffer.substring(0, 2).toInt()) ;  // 時
                rtc.setMinut(TimeBuffer.substring(3, 5).toInt()) ; // 分
                rtc.setSecond(TimeBuffer.substring(6, 8).toInt()) ;// 秒
                skRTC.SetWeekdays(WeekDay) ;                       // 曜日
                rtc.startClock() ;// RTCのカウントを開始する
                ProcFlag = 9 ;    // 日付時刻の設定完了
            }
        }
    }
    // バッファ内容をOLEDに送る
    u8g2.sendBuffer() ;

    delay(100) ;
}

////////////////////////////////////////////////////////////////////////////////
// ＲＴＣの現在の時間を読み出します。
void GetTime() {
    Time nowTime = rtc.getTime() ;
    sprintf(WorkBuffer,"%d/%02d/%02d",nowTime.year+2000,nowTime.month,nowTime.day) ;
    DateBuffer = WorkBuffer ;   // 日付の設定
    sprintf(WorkBuffer,"%02d:%02d:%02d",nowTime.hour,nowTime.minute,nowTime.second) ;
    TimeBuffer = WorkBuffer ;   // 時刻の設定
    WeekDay = nowTime.weekday ; // 曜日の設定
}
