/*******************************************************************************
*  skPCF8563 - PCF8563(RTC)用関数ライブラリ                                    *
*                                                                              *
*    SetWeekdays     - 曜日の設定を行う処理                                    *
*    SetAlarmWeekday - アラーム発生時の曜日を設定する処理                      *
*    SetAlarmDay     - アラーム発生時の日を設定する処理                        *
*    SetAlarmHour    - アラーム発生時の時を設定する処理                        *
*    SetAlarmMinute  - アラーム発生時のを分設定する処理                        *
*    SetAlarmTime    - アラーム発生時の時刻を設定し、アラームを有効にする処理  *
*    AlarmMatch      - アラームの発生状況をポーリングする処理                  *
*    AlarmEnable     - アラームの発生を有効にする処理                          *
*    AlarmDisable    - アラームの発生を無効にする処理                          *
*    AlarmClearFlag  - アラームフラグをクリアにする処理                        *
*    TimerCountdownStart - タイマーのカウントダウンを開始する処理              *
*    TimerCountdownCheck - タイマーのカウントダウンをポーリングする処理        *
*    TimerDisable        - カウントダウンを無効にする処理                      *
*    TimerClearFlag      - タイマーフラグをクリアにする処理                    *
*    write - PCF8563のレジスタに1バイトのデータを書き込む処理                  *
*    read  - PCF8563のレジスタから1バイトのデータを読み取る処理                *
*                                                                              *
*  PCF8563(https://github.com/Bill2462/PCF8563-Arduino-Library)のライブラリと  *
*  一緒に利用する様に設計されています。                                        *
*  又、INTピン割り込み出力は考慮されていません。                               *
* ============================================================================ *
*   VERSION  DATE        BY             CHANGE/COMMENT                         *
* ---------------------------------------------------------------------------- *
*   1.00     2021-01-26  きむ茶工房     Create                                 *
* ============================================================================ *
*   Arduino IDE 1.8.12 (Ardino Duemilanove 328/UNO/Seeeduino XIAO)             *
*******************************************************************************/
#include "skPCF8563.h"


skPCF8563::skPCF8563(void)
{
//    Wire.begin() ;// initialize the I2C interface
}

// PCF8563に曜日を設定します
// 引数
//  ・ uint8_t wdays : 曜日の値を指定します(0:日曜～6:土曜)
// Returns
//  ・ 無し
void skPCF8563::SetWeekdays(uint8_t wdays)
{
    const uint8_t data = (((wdays/10) << 4)|(wdays%10)) ;
    write(PCF8563_Weekdays,data) ;
}

// PCF8563にアラーム発生の曜日を設定します
// 引数
//  ・ uint8_t wday : アラームを発生させる曜日の値を指定します(0:日曜～6:土曜 or 80)
// Returns
//  ・ 無し
void skPCF8563::SetAlarmWeekday(uint8_t wday)
{
    const uint8_t data = (((wday/10) << 4)|(wday%10)) ;
    write(PCF8563_AlarmWeekday,data) ;
}

// PCF8563にアラーム発生の日を設定します
// 引数
//  ・ uint8_t day : アラームを発生させる日の値を指定します(01-31 or 80)
// Returns
//  ・ 無し
void skPCF8563::SetAlarmDay(uint8_t day)
{
    const uint8_t data = (((day/10) << 4)|(day%10)) ;
    write(PCF8563_AlarmDay,data) ;
}

// PCF8563にアラーム発生の時を設定します
// 引数
//  ・ uint8_t hour : アラームを発生させる時の値を指定します(00-23 or 80)
// Returns
//  ・ 無し
void skPCF8563::SetAlarmHour(uint8_t hour)
{
    const uint8_t data = (((hour/10) << 4)|(hour%10)) ;
    write(PCF8563_AlarmHour,data) ;
}

// PCF8563にアラーム発生の分を設定します
// 引数
//  ・ uint8_t minute : アラームを発生させる分の値を指定します(00-59 or 80)
// Returns
//  ・ 無し
void skPCF8563::SetAlarmMinute(uint8_t minute)
{
    const uint8_t data = (((minute/10) << 4)|(minute%10)) ;
    write(PCF8563_AlarmMinute,data) ;
}

// PCF8563にアラーム発生の時刻を設定し、アラームを有効にします。
// 利用しない時間帯は"80"を必ず設定します、例えば毎30分(1時間毎に起動)に起動させるなら
// ???.SetAlarmTime(30, 80, 80, 80);とします。
// 引数
//  ・ uint8_t minute : アラームを発生させる分の値を指定します(00-59 or 80)
//  ・ uint8_t hour   : アラームを発生させる時の値を指定します(00-23 or 80)
//  ・ uint8_t day    : アラームを発生させる日の値を指定します(01-31 or 80)
//  ・ uint8_t wday   : アラームを発生させる曜日の値を指定します(0:日曜～6:土曜 or 80)
// Returns
//  ・ 無し
void skPCF8563::SetAlarmTime(uint8_t minute,uint8_t hour,uint8_t day,uint8_t wday)
{
    SetAlarmMinute(minute) ;
    SetAlarmHour(hour) ;
    SetAlarmDay(day) ;
    SetAlarmWeekday(wday) ;
    AlarmEnable() ;           // アラーム有効
}

// アラームの発生状況をポーリングします(常にループさせます)
// 引数
//  ・ 無し
// Returns
//  ・ uint8_t　0=アラームの発生は無し 1=アラームの発生は有り
uint8_t skPCF8563::AlarmMatch()
{
    uint8_t c = read(PCF8563_CTL_STS2) ;
    return ((c >> 3) & 0x01) ;
}

// PCF8563のアラーム発生を有効にします
// 引数
//  ・ 無し
// Returns
//  ・ 無し
void skPCF8563::AlarmEnable()
{
    uint8_t c = read(PCF8563_CTL_STS2) ;
    c = c | PCF8563_AIE ;
    write(PCF8563_CTL_STS2,c) ;
}

// PCF8563のアラーム発生を無効にします
// 引数
//  ・ 無し
// Returns
//  ・ 無し
void skPCF8563::AlarmDisable()
{
    uint8_t c = read(PCF8563_CTL_STS2) ;
    c = c & ~PCF8563_AIE ;
    write(PCF8563_CTL_STS2,c) ;
}

// PCF8563のアラームフラグをクリアします
// 引数
//  ・ 無し
// Returns
//  ・ 無し
void skPCF8563::AlarmClearFlag()
{
    uint8_t c = read(PCF8563_CTL_STS2) ;
    c = c & ~PCF8563_AF ;
    write(PCF8563_CTL_STS2,c) ;
}

// PCF8563のタイマーのカウントダウンを開始します。
// 引数
//  ・ uint8_t timer : カウントダウンの値を指定します(0-255)
//  ・ uint8_t frequency : タイマーソースクロック周波数を指定します
//                         PCF8563_TimerF_1_60Hz = １分間に1回カウントダウン
//                         PCF8563_TimerF_1Hz    = １秒間に1回カウントダウン
// Returns
//  ・ 無し
void skPCF8563::TimerCountdownStart(uint8_t timer,uint8_t frequency)
{
    write(PCF8563_TimerValue,timer) ;   // カウントダウンの値設定
    write(PCF8563_Timer_CTL,frequency|0x80) ; // タイマーの制御レジスタ値設定
    uint8_t c = read(PCF8563_CTL_STS2) ;
    c = c | PCF8563_TIE ;               // タイマーのカウントダウン有効
    write(PCF8563_CTL_STS2,c) ;
}

// タイマーのカウントダウンをポーリングします(常にループさせます)
// 引数
//  ・ 無し
// Returns
//  ・ uint8_t　0=カウントダウンしていない 1=カウントダウンした
uint8_t skPCF8563::TimerCountdownCheck()
{
    uint8_t c = read(PCF8563_CTL_STS2) ;
    return ((c >> 2) & 0x01) ;
}

// PCF8563のタイマーのカウントダウンを無効にします
// 引数
//  ・ 無し
// Returns
//  ・ 無し
void skPCF8563::TimerDisable()
{
    uint8_t c = read(PCF8563_CTL_STS2) ;
    c = c & ~PCF8563_TIE ;              // タイマーのカウントダウン無効
    write(PCF8563_CTL_STS2,c) ;
    write(PCF8563_Timer_CTL,PCF8563_TimerF_1_60Hz) ; // クロック周波数値は1/60Hz
}

// PCF8563のタイマーフラグをクリアします
// 引数
//  ・ 無し
// Returns
//  ・ 無し
void skPCF8563::TimerClearFlag()
{
    uint8_t c = read(PCF8563_CTL_STS2) ;
    c = c & ~PCF8563_TF ;
    write(PCF8563_CTL_STS2,c) ;
}

// PCF8563のレジスタに1バイトのデータを書き込む
// 引数
//  ・ uint8_t address : 書き込みを行うレジスタのアドレスを指定します
//  ・ uint8_t data    : レジスタに書き込みたいデータのバイト
// Returns
//  ・ 無し
void skPCF8563::write(uint8_t address, uint8_t data)
{
    Wire.beginTransmission(PCF8563_address);
    Wire.write(address);
    Wire.write(data);
    Wire.endTransmission();
}

// PCF8563のレジスタから1バイトのデータを読み取る
// 引数
//  ・ uint8_t address  - 読み込みを行うレジスタのアドレスを指定します
// Returns
//  ・ uint8_t　レジスタから読み取られたデータのバイト
uint8_t skPCF8563::read(uint8_t address)
{
  Wire.beginTransmission(PCF8563_address) ;
  Wire.write(address) ;
  Wire.endTransmission() ;
  Wire.requestFrom(PCF8563_address,1) ;
  uint8_t data = Wire.read() ;

  return data;
}
